@@ -4,9 +4,9 @@ import (
 	"strings"
 	"testing"
 
+	"github.com/evanw/esbuild/internal/ast"
 	"github.com/evanw/esbuild/internal/compat"
 	"github.com/evanw/esbuild/internal/config"
-	"github.com/evanw/esbuild/internal/js_ast"
 	"github.com/evanw/esbuild/internal/js_parser"
 	"github.com/evanw/esbuild/internal/logger"
 	"github.com/evanw/esbuild/internal/renamer"
@@ -35,7 +35,7 @@ func expectPrintedCommon(t *testing.T, name string, contents string, expected st
 		if !ok {
 			t.Fatal("Parse error")
 		}
-		symbols := js_ast.NewSymbolMap(1)
+		symbols := ast.NewSymbolMap(1)
 		symbols.SymbolsForSource[0] = tree.Symbols
 		r := renamer.NewNoOpRenamer(symbols)
 		js := Print(tree, symbols, r, Options{
@@ -93,8 +93,8 @@ func expectPrintedMinifyASCII(t *testing.T, contents string, expected string) {
 func expectPrintedTarget(t *testing.T, esVersion int, contents string, expected string) {
 	t.Helper()
 	expectPrintedCommon(t, contents, contents, expected, config.Options{
-		UnsupportedJSFeatures: compat.UnsupportedJSFeatures(map[compat.Engine][]int{
-			compat.ES: {esVersion},
+		UnsupportedJSFeatures: compat.UnsupportedJSFeatures(map[compat.Engine]compat.Semver{
+			compat.ES: {Parts: []int{esVersion}},
 		}),
 	})
 }
@@ -102,8 +102,8 @@ func expectPrintedTarget(t *testing.T, esVersion int, contents string, expected
 func expectPrintedTargetMinify(t *testing.T, esVersion int, contents string, expected string) {
 	t.Helper()
 	expectPrintedCommon(t, contents+" [minified]", contents, expected, config.Options{
-		UnsupportedJSFeatures: compat.UnsupportedJSFeatures(map[compat.Engine][]int{
-			compat.ES: {esVersion},
+		UnsupportedJSFeatures: compat.UnsupportedJSFeatures(map[compat.Engine]compat.Semver{
+			compat.ES: {Parts: []int{esVersion}},
 		}),
 		MinifyWhitespace: true,
 	})
@@ -112,8 +112,8 @@ func expectPrintedTargetMinify(t *testing.T, esVersion int, contents string, exp
 func expectPrintedTargetMangle(t *testing.T, esVersion int, contents string, expected string) {
 	t.Helper()
 	expectPrintedCommon(t, contents+" [mangled]", contents, expected, config.Options{
-		UnsupportedJSFeatures: compat.UnsupportedJSFeatures(map[compat.Engine][]int{
-			compat.ES: {esVersion},
+		UnsupportedJSFeatures: compat.UnsupportedJSFeatures(map[compat.Engine]compat.Semver{
+			compat.ES: {Parts: []int{esVersion}},
 		}),
 		MinifySyntax: true,
 	})
@@ -122,8 +122,8 @@ func expectPrintedTargetMangle(t *testing.T, esVersion int, contents string, exp
 func expectPrintedTargetASCII(t *testing.T, esVersion int, contents string, expected string) {
 	t.Helper()
 	expectPrintedCommon(t, contents+" [ascii]", contents, expected, config.Options{
-		UnsupportedJSFeatures: compat.UnsupportedJSFeatures(map[compat.Engine][]int{
-			compat.ES: {esVersion},
+		UnsupportedJSFeatures: compat.UnsupportedJSFeatures(map[compat.Engine]compat.Semver{
+			compat.ES: {Parts: []int{esVersion}},
 		}),
 		ASCIIOnly: true,
 	})
@@ -571,14 +571,14 @@ func TestCommentsAndParentheses(t *testing.T) {
 
 func TestPureComment(t *testing.T) {
 	expectPrinted(t,
-		"(function() {})",
-		"(function() {\n});\n")
+		"(function() { foo() })",
+		"(function() {\n  foo();\n});\n")
 	expectPrinted(t,
-		"(function() {})()",
-		"(function() {\n})();\n")
+		"(function() { foo() })()",
+		"(function() {\n  foo();\n})();\n")
 	expectPrinted(t,
-		"/*@__PURE__*/(function() {})()",
-		"/* @__PURE__ */ (function() {\n})();\n")
+		"/*@__PURE__*/(function() { foo() })()",
+		"/* @__PURE__ */ (function() {\n  foo();\n})();\n")
 
 	expectPrinted(t,
 		"new (function() {})",
@@ -591,14 +591,14 @@ func TestPureComment(t *testing.T) {
 		"/* @__PURE__ */ new function() {\n}();\n")
 
 	expectPrinted(t,
-		"export default (function() {})",
-		"export default (function() {\n});\n")
+		"export default (function() { foo() })",
+		"export default (function() {\n  foo();\n});\n")
 	expectPrinted(t,
-		"export default (function() {})()",
-		"export default (function() {\n})();\n")
+		"export default (function() { foo() })()",
+		"export default (function() {\n  foo();\n})();\n")
 	expectPrinted(t,
-		"export default /*@__PURE__*/(function() {})()",
-		"export default /* @__PURE__ */ (function() {\n})();\n")
+		"export default /*@__PURE__*/(function() { foo() })()",
+		"export default /* @__PURE__ */ (function() {\n  foo();\n})();\n")
 }
 
 func TestGenerator(t *testing.T) {
@@ -695,6 +695,13 @@ func TestClass(t *testing.T) {
 	expectPrinted(t, "class Foo { static set foo(x) {} }", "class Foo {\n  static set foo(x) {\n  }\n}\n")
 }
 
+func TestAutoAccessors(t *testing.T) {
+	expectPrinted(t, "class Foo { accessor x; static accessor y }", "class Foo {\n  accessor x;\n  static accessor y;\n}\n")
+	expectPrinted(t, "class Foo { accessor [x]; static accessor [y] }", "class Foo {\n  accessor [x];\n  static accessor [y];\n}\n")
+	expectPrintedMinify(t, "class Foo { accessor x; static accessor y }", "class Foo{accessor x;static accessor y}")
+	expectPrintedMinify(t, "class Foo { accessor [x]; static accessor [y] }", "class Foo{accessor[x];static accessor[y]}")
+}
+
 func TestPrivateIdentifiers(t *testing.T) {
 	expectPrinted(t, "class Foo { #foo; foo() { return #foo in this } }", "class Foo {\n  #foo;\n  foo() {\n    return #foo in this;\n  }\n}\n")
 	expectPrintedMinify(t, "class Foo { #foo; foo() { return #foo in this } }", "class Foo{#foo;foo(){return#foo in this}}")
@@ -847,7 +854,7 @@ func TestMinify(t *testing.T) {
 	expectPrinted(t, "x = '\\n'", "x = \"\\n\";\n")
 	expectPrintedMangle(t, "x = '\\n'", "x = `\n`;\n")
 	expectPrintedMangle(t, "x = {'\\n': 0}", "x = { \"\\n\": 0 };\n")
-	expectPrintedMangle(t, "(class{'\\n' = 0})", "(class {\n  \"\\n\" = 0;\n});\n")
+	expectPrintedMangle(t, "x = class{'\\n' = 0}", "x = class {\n  \"\\n\" = 0;\n};\n")
 	expectPrintedMangle(t, "class Foo{'\\n' = 0}", "class Foo {\n  \"\\n\" = 0;\n}\n")
 
 	// Special identifiers must not be minified
@@ -1096,3 +1103,16 @@ func TestBinaryOperatorVisitor(t *testing.T) {
 	x := "x = f()" + strings.Repeat(" || f()", 10_000) + ";\n"
 	expectPrinted(t, x, x)
 }
+
+// See: https://github.com/tc39/proposal-explicit-resource-management
+func TestUsing(t *testing.T) {
+	expectPrinted(t, "using x = y", "using x = y;\n")
+	expectPrinted(t, "using x = y, z = _", "using x = y, z = _;\n")
+	expectPrintedMinify(t, "using x = y", "using x=y;")
+	expectPrintedMinify(t, "using x = y, z = _", "using x=y,z=_;")
+
+	expectPrinted(t, "await using x = y", "await using x = y;\n")
+	expectPrinted(t, "await using x = y, z = _", "await using x = y, z = _;\n")
+	expectPrintedMinify(t, "await using x = y", "await using x=y;")
+	expectPrintedMinify(t, "await using x = y, z = _", "await using x=y,z=_;")
+}
