@@ -17,10 +17,12 @@ const (
 	MsgID_JS_AssignToDefine
 	MsgID_JS_AssignToImport
 	MsgID_JS_CallImportNamespace
+	MsgID_JS_ClassNameWillThrow
 	MsgID_JS_CommonJSVariableInESM
 	MsgID_JS_DeleteSuperProperty
 	MsgID_JS_DirectEval
 	MsgID_JS_DuplicateCase
+	MsgID_JS_DuplicateClassMember
 	MsgID_JS_DuplicateObjectKey
 	MsgID_JS_EmptyImportMeta
 	MsgID_JS_EqualsNaN
@@ -46,6 +48,7 @@ const (
 	MsgID_CSS_InvalidAtLayer
 	MsgID_CSS_InvalidCalc
 	MsgID_CSS_JSCommentInCSS
+	MsgID_CSS_UndefinedComposesFrom
 	MsgID_CSS_UnsupportedAtCharset
 	MsgID_CSS_UnsupportedAtNamespace
 	MsgID_CSS_UnsupportedCSSProperty
@@ -54,6 +57,7 @@ const (
 	// Bundler
 	MsgID_Bundler_AmbiguousReexport
 	MsgID_Bundler_DifferentPathCase
+	MsgID_Bundler_EmptyGlob
 	MsgID_Bundler_IgnoredBareImport
 	MsgID_Bundler_IgnoredDynamicImport
 	MsgID_Bundler_ImportIsUndefined
@@ -74,15 +78,15 @@ const (
 	MsgID_PackageJSON_LAST // Keep this last
 
 	// tsconfig.json
-	MsgID_TsconfigJSON_FIRST // Keep this first
-	MsgID_TsconfigJSON_Cycle
-	MsgID_TsconfigJSON_InvalidImportsNotUsedAsValues
-	MsgID_TsconfigJSON_InvalidJSX
-	MsgID_TsconfigJSON_InvalidModuleSuffixes
-	MsgID_TsconfigJSON_InvalidPaths
-	MsgID_TsconfigJSON_InvalidTarget
-	MsgID_TsconfigJSON_Missing
-	MsgID_TsconfigJSON_LAST // Keep this last
+	MsgID_TSConfigJSON_FIRST // Keep this first
+	MsgID_TSConfigJSON_Cycle
+	MsgID_TSConfigJSON_InvalidImportsNotUsedAsValues
+	MsgID_TSConfigJSON_InvalidJSX
+	MsgID_TSConfigJSON_InvalidPaths
+	MsgID_TSConfigJSON_InvalidTarget
+	MsgID_TSConfigJSON_InvalidTopLevelOption
+	MsgID_TSConfigJSON_Missing
+	MsgID_TSConfigJSON_LAST // Keep this last
 
 	MsgID_END // Keep this at the end (used only for tests)
 )
@@ -100,6 +104,8 @@ func StringToMsgIDs(str string, logLevel LogLevel, overrides map[MsgID]LogLevel)
 		overrides[MsgID_JS_AssignToImport] = logLevel
 	case "call-import-namespace":
 		overrides[MsgID_JS_CallImportNamespace] = logLevel
+	case "class-name-will-throw":
+		overrides[MsgID_JS_ClassNameWillThrow] = logLevel
 	case "commonjs-variable-in-esm":
 		overrides[MsgID_JS_CommonJSVariableInESM] = logLevel
 	case "delete-super-property":
@@ -108,6 +114,8 @@ func StringToMsgIDs(str string, logLevel LogLevel, overrides map[MsgID]LogLevel)
 		overrides[MsgID_JS_DirectEval] = logLevel
 	case "duplicate-case":
 		overrides[MsgID_JS_DuplicateCase] = logLevel
+	case "duplicate-class-member":
+		overrides[MsgID_JS_DuplicateClassMember] = logLevel
 	case "duplicate-object-key":
 		overrides[MsgID_JS_DuplicateObjectKey] = logLevel
 	case "empty-import-meta":
@@ -156,6 +164,8 @@ func StringToMsgIDs(str string, logLevel LogLevel, overrides map[MsgID]LogLevel)
 		overrides[MsgID_CSS_InvalidCalc] = logLevel
 	case "js-comment-in-css":
 		overrides[MsgID_CSS_JSCommentInCSS] = logLevel
+	case "undefined-composes-from":
+		overrides[MsgID_CSS_UndefinedComposesFrom] = logLevel
 	case "unsupported-@charset":
 		overrides[MsgID_CSS_UnsupportedAtCharset] = logLevel
 	case "unsupported-@namespace":
@@ -170,6 +180,8 @@ func StringToMsgIDs(str string, logLevel LogLevel, overrides map[MsgID]LogLevel)
 		overrides[MsgID_Bundler_AmbiguousReexport] = logLevel
 	case "different-path-case":
 		overrides[MsgID_Bundler_DifferentPathCase] = logLevel
+	case "empty-glob":
+		overrides[MsgID_Bundler_EmptyGlob] = logLevel
 	case "ignored-bare-import":
 		overrides[MsgID_Bundler_IgnoredBareImport] = logLevel
 	case "ignored-dynamic-import":
@@ -195,7 +207,7 @@ func StringToMsgIDs(str string, logLevel LogLevel, overrides map[MsgID]LogLevel)
 		}
 
 	case "tsconfig.json":
-		for i := MsgID_TsconfigJSON_FIRST; i <= MsgID_TsconfigJSON_LAST; i++ {
+		for i := MsgID_TSConfigJSON_FIRST; i <= MsgID_TSConfigJSON_LAST; i++ {
 			overrides[i] = logLevel
 		}
 
@@ -218,6 +230,8 @@ func MsgIDToString(id MsgID) string {
 		return "assign-to-import"
 	case MsgID_JS_CallImportNamespace:
 		return "call-import-namespace"
+	case MsgID_JS_ClassNameWillThrow:
+		return "class-name-will-throw"
 	case MsgID_JS_CommonJSVariableInESM:
 		return "commonjs-variable-in-esm"
 	case MsgID_JS_DeleteSuperProperty:
@@ -226,6 +240,8 @@ func MsgIDToString(id MsgID) string {
 		return "direct-eval"
 	case MsgID_JS_DuplicateCase:
 		return "duplicate-case"
+	case MsgID_JS_DuplicateClassMember:
+		return "duplicate-class-member"
 	case MsgID_JS_DuplicateObjectKey:
 		return "duplicate-object-key"
 	case MsgID_JS_EmptyImportMeta:
@@ -274,6 +290,8 @@ func MsgIDToString(id MsgID) string {
 		return "invalid-calc"
 	case MsgID_CSS_JSCommentInCSS:
 		return "js-comment-in-css"
+	case MsgID_CSS_UndefinedComposesFrom:
+		return "undefined-composes-from"
 	case MsgID_CSS_UnsupportedAtCharset:
 		return "unsupported-@charset"
 	case MsgID_CSS_UnsupportedAtNamespace:
@@ -288,6 +306,8 @@ func MsgIDToString(id MsgID) string {
 		return "ambiguous-reexport"
 	case MsgID_Bundler_DifferentPathCase:
 		return "different-path-case"
+	case MsgID_Bundler_EmptyGlob:
+		return "empty-glob"
 	case MsgID_Bundler_IgnoredBareImport:
 		return "ignored-bare-import"
 	case MsgID_Bundler_IgnoredDynamicImport:
@@ -311,7 +331,7 @@ func MsgIDToString(id MsgID) string {
 		if id >= MsgID_PackageJSON_FIRST && id <= MsgID_PackageJSON_LAST {
 			return "package.json"
 		}
-		if id >= MsgID_TsconfigJSON_FIRST && id <= MsgID_TsconfigJSON_LAST {
+		if id >= MsgID_TSConfigJSON_FIRST && id <= MsgID_TSConfigJSON_LAST {
 			return "tsconfig.json"
 		}
 	}
