@@ -5,6 +5,8 @@ package api
 
 import (
 	"bytes"
+	"encoding/base64"
+	"encoding/binary"
 	"errors"
 	"fmt"
 	"io/ioutil"
@@ -25,6 +27,7 @@ import (
 	"github.com/evanw/esbuild/internal/cache"
 	"github.com/evanw/esbuild/internal/compat"
 	"github.com/evanw/esbuild/internal/config"
+	"github.com/evanw/esbuild/internal/css_ast"
 	"github.com/evanw/esbuild/internal/fs"
 	"github.com/evanw/esbuild/internal/graph"
 	"github.com/evanw/esbuild/internal/helpers"
@@ -250,12 +253,16 @@ func validateLoader(value Loader) config.Loader {
 		return config.LoaderEmpty
 	case LoaderFile:
 		return config.LoaderFile
+	case LoaderGlobalCSS:
+		return config.LoaderGlobalCSS
 	case LoaderJS:
 		return config.LoaderJS
 	case LoaderJSON:
 		return config.LoaderJSON
 	case LoaderJSX:
 		return config.LoaderJSX
+	case LoaderLocalCSS:
+		return config.LoaderLocalCSS
 	case LoaderNone:
 		return config.LoaderNone
 	case LoaderText:
@@ -288,41 +295,36 @@ func validateEngine(value EngineName) compat.Engine {
 	}
 }
 
-var versionRegex = regexp.MustCompile(`^([0-9]+)(?:\.([0-9]+))?(?:\.([0-9]+))?$`)
-var preReleaseVersionRegex = regexp.MustCompile(`^([0-9]+)(?:\.([0-9]+))?(?:\.([0-9]+))?-`)
+var versionRegex = regexp.MustCompile(`^([0-9]+)(?:\.([0-9]+))?(?:\.([0-9]+))?(-[A-Za-z0-9]+(?:\.[A-Za-z0-9]+)*)?$`)
 
-func validateFeatures(log logger.Log, target Target, engines []Engine) (config.TargetFromAPI, compat.JSFeature, compat.CSSFeature, string) {
+func validateFeatures(log logger.Log, target Target, engines []Engine) (compat.JSFeature, compat.CSSFeature, map[css_ast.D]compat.CSSPrefix, string) {
 	if target == DefaultTarget && len(engines) == 0 {
-		return config.TargetWasUnconfigured, 0, 0, ""
+		return 0, 0, nil, ""
 	}
 
-	constraints := make(map[compat.Engine][]int)
+	constraints := make(map[compat.Engine]compat.Semver)
 	targets := make([]string, 0, 1+len(engines))
-	targetFromAPI := config.TargetWasConfigured
 
 	switch target {
 	case ES5:
-		constraints[compat.ES] = []int{5}
+		constraints[compat.ES] = compat.Semver{Parts: []int{5}}
 	case ES2015:
-		constraints[compat.ES] = []int{2015}
+		constraints[compat.ES] = compat.Semver{Parts: []int{2015}}
 	case ES2016:
-		constraints[compat.ES] = []int{2016}
+		constraints[compat.ES] = compat.Semver{Parts: []int{2016}}
 	case ES2017:
-		constraints[compat.ES] = []int{2017}
+		constraints[compat.ES] = compat.Semver{Parts: []int{2017}}
 	case ES2018:
-		constraints[compat.ES] = []int{2018}
+		constraints[compat.ES] = compat.Semver{Parts: []int{2018}}
 	case ES2019:
-		constraints[compat.ES] = []int{2019}
+		constraints[compat.ES] = compat.Semver{Parts: []int{2019}}
 	case ES2020:
-		constraints[compat.ES] = []int{2020}
+		constraints[compat.ES] = compat.Semver{Parts: []int{2020}}
 	case ES2021:
-		constraints[compat.ES] = []int{2021}
+		constraints[compat.ES] = compat.Semver{Parts: []int{2021}}
 	case ES2022:
-		constraints[compat.ES] = []int{2022}
-		targetFromAPI = config.TargetWasConfiguredAndAtLeastES2022
-	case ESNext:
-		targetFromAPI = config.TargetWasConfiguredAndAtLeastES2022
-	case DefaultTarget:
+		constraints[compat.ES] = compat.Semver{Parts: []int{2022}}
+	case ESNext, DefaultTarget:
 	default:
 		panic("Invalid target")
 	}
@@ -330,47 +332,38 @@ func validateFeatures(log logger.Log, target Target, engines []Engine) (config.T
 	for _, engine := range engines {
 		if match := versionRegex.FindStringSubmatch(engine.Version); match != nil {
 			if major, err := strconv.Atoi(match[1]); err == nil {
-				version := []int{major}
+				parts := []int{major}
 				if minor, err := strconv.Atoi(match[2]); err == nil {
-					version = append(version, minor)
+					parts = append(parts, minor)
+					if patch, err := strconv.Atoi(match[3]); err == nil {
+						parts = append(parts, patch)
+					}
 				}
-				if patch, err := strconv.Atoi(match[3]); err == nil {
-					version = append(version, patch)
+				constraints[convertEngineName(engine.Name)] = compat.Semver{
+					Parts:      parts,
+					PreRelease: match[4],
 				}
-				constraints[convertEngineName(engine.Name)] = version
 				continue
 			}
 		}
 
 		text := "All version numbers passed to esbuild must be in the format \"X\", \"X.Y\", or \"X.Y.Z\" where X, Y, and Z are non-negative integers."
 
-		// Our internal version-to-feature database only includes version triples.
-		// We don't have any data on pre-release versions, so we don't accept them.
-		if preReleaseVersionRegex.MatchString(engine.Version) {
-			text += " Pre-release versions are not supported and cannot be used."
-		}
-
 		log.AddErrorWithNotes(nil, logger.Range{}, fmt.Sprintf("Invalid version: %q", engine.Version),
 			[]logger.MsgData{{Text: text}})
 	}
 
 	for engine, version := range constraints {
-		var text string
-		switch len(version) {
-		case 1:
-			text = fmt.Sprintf("%s%d", engine.String(), version[0])
-		case 2:
-			text = fmt.Sprintf("%s%d.%d", engine.String(), version[0], version[1])
-		case 3:
-			text = fmt.Sprintf("%s%d.%d.%d", engine.String(), version[0], version[1], version[2])
-		}
-		targets = append(targets, text)
+		targets = append(targets, engine.String()+version.String())
+	}
+	if target == ESNext {
+		targets = append(targets, "esnext")
 	}
 
 	sort.Strings(targets)
 	targetEnv := helpers.StringArrayToQuotedCommaSeparatedString(targets)
 
-	return targetFromAPI, compat.UnsupportedJSFeatures(constraints), compat.UnsupportedCSSFeatures(constraints), targetEnv
+	return compat.UnsupportedJSFeatures(constraints), compat.UnsupportedCSSFeatures(constraints), compat.CSSPrefixData(constraints), targetEnv
 }
 
 func validateSupported(log logger.Log, supported map[string]bool) (
@@ -981,12 +974,16 @@ type internalContext struct {
 	args          rebuildArgs
 	activeBuild   *buildInProgress
 	recentBuild   *BuildResult
-	latestSummary buildSummary
 	realFS        fs.FS
 	absWorkingDir string
 	watcher       *watcher
 	handler       *apiHandler
 	didDispose    bool
+
+	// This saves just enough information to be able to compute a useful diff
+	// between two sets of output files. That way we don't need to hold both
+	// sets of output files in memory at once to compute a diff.
+	latestHashes map[string]string
 }
 
 func (ctx *internalContext) rebuild() rebuildState {
@@ -1012,14 +1009,15 @@ func (ctx *internalContext) rebuild() rebuildState {
 	args := ctx.args
 	watcher := ctx.watcher
 	handler := ctx.handler
-	oldSummary := ctx.latestSummary
+	oldHashes := ctx.latestHashes
 	args.options.CancelFlag = &build.cancel
 	ctx.mutex.Unlock()
 
 	// Do the build without holding the mutex
-	build.state = rebuildImpl(args, oldSummary)
+	var newHashes map[string]string
+	build.state, newHashes = rebuildImpl(args, oldHashes)
 	if handler != nil {
-		handler.broadcastBuildResult(build.state.result, build.state.summary)
+		handler.broadcastBuildResult(build.state.result, newHashes)
 	}
 	if watcher != nil {
 		watcher.setWatchData(build.state.watchData)
@@ -1030,7 +1028,7 @@ func (ctx *internalContext) rebuild() rebuildState {
 	ctx.mutex.Lock()
 	ctx.activeBuild = nil
 	ctx.recentBuild = recentBuild
-	ctx.latestSummary = build.state.summary
+	ctx.latestHashes = newHashes
 	ctx.mutex.Unlock()
 
 	// Clear the recent build after it goes stale
@@ -1095,8 +1093,11 @@ func (ctx *internalContext) Watch(options WatchOptions) error {
 		return errors.New("Watch mode has already been enabled")
 	}
 
+	logLevel := ctx.args.logOptions.LogLevel
 	ctx.watcher = &watcher{
-		fs: ctx.realFS,
+		fs:        ctx.realFS,
+		shouldLog: logLevel == logger.LevelInfo || logLevel == logger.LevelDebug || logLevel == logger.LevelVerbose,
+		useColor:  ctx.args.logOptions.Color,
 		rebuild: func() fs.WatchData {
 			return ctx.rebuild().watchData
 		},
@@ -1106,7 +1107,7 @@ func (ctx *internalContext) Watch(options WatchOptions) error {
 	ctx.args.options.WatchMode = true
 
 	// Start the file watcher goroutine
-	ctx.watcher.start(ctx.args.logOptions.LogLevel, ctx.args.logOptions.Color)
+	ctx.watcher.start()
 
 	// Do the first watch mode build on another goroutine
 	go func() {
@@ -1244,7 +1245,7 @@ func validateBuildOptions(
 	options config.Options,
 	entryPoints []bundler.EntryPoint,
 ) {
-	targetFromAPI, jsFeatures, cssFeatures, targetEnv := validateFeatures(log, buildOpts.Target, buildOpts.Engines)
+	jsFeatures, cssFeatures, cssPrefixData, targetEnv := validateFeatures(log, buildOpts.Target, buildOpts.Engines)
 	jsOverrides, jsMask, cssOverrides, cssMask := validateSupported(log, buildOpts.Supported)
 	outJS, outCSS := validateOutputExtensions(log, buildOpts.OutExtension)
 	bannerJS, bannerCSS := validateBannerOrFooter(log, "banner", buildOpts.Banner)
@@ -1253,7 +1254,7 @@ func validateBuildOptions(
 	platform := validatePlatform(buildOpts.Platform)
 	defines, injectedDefines := validateDefines(log, buildOpts.Define, buildOpts.Pure, platform, true /* isBuildAPI */, minify, buildOpts.Drop)
 	options = config.Options{
-		TargetFromAPI:                      targetFromAPI,
+		CSSPrefixData:                      cssPrefixData,
 		UnsupportedJSFeatures:              jsFeatures.ApplyOverrides(jsOverrides, jsMask),
 		UnsupportedCSSFeatures:             cssFeatures.ApplyOverrides(cssOverrides, cssMask),
 		UnsupportedJSFeatureOverrides:      jsOverrides,
@@ -1280,9 +1281,11 @@ func validateBuildOptions(
 		MinifySyntax:          buildOpts.MinifySyntax,
 		MinifyWhitespace:      buildOpts.MinifyWhitespace,
 		MinifyIdentifiers:     buildOpts.MinifyIdentifiers,
+		LineLimit:             buildOpts.LineLimit,
 		MangleProps:           validateRegex(log, "mangle props", buildOpts.MangleProps),
 		ReserveProps:          validateRegex(log, "reserve props", buildOpts.ReserveProps),
 		MangleQuoted:          buildOpts.MangleQuoted == MangleQuotedTrue,
+		DropLabels:            append([]string{}, buildOpts.DropLabels...),
 		DropDebugger:          (buildOpts.Drop & DropDebugger) != 0,
 		AllowOverwrite:        buildOpts.AllowOverwrite,
 		ASCIIOnly:             validateASCIIOnly(buildOpts.Charset),
@@ -1305,7 +1308,8 @@ func validateBuildOptions(
 		ExternalSettings:      validateExternals(log, realFS, buildOpts.External),
 		ExternalPackages:      buildOpts.Packages == PackagesExternal,
 		PackageAliases:        validateAlias(log, realFS, buildOpts.Alias),
-		TsConfigOverride:      validatePath(log, realFS, buildOpts.Tsconfig, "tsconfig path"),
+		TSConfigPath:          validatePath(log, realFS, buildOpts.Tsconfig, "tsconfig path"),
+		TSConfigRaw:           buildOpts.TsconfigRaw,
 		MainFields:            buildOpts.MainFields,
 		PublicPath:            buildOpts.PublicPath,
 		KeepNames:             buildOpts.KeepNames,
@@ -1327,11 +1331,18 @@ func validateBuildOptions(
 		options.AbsNodePaths[i] = validatePath(log, realFS, path, "node path")
 	}
 	entryPoints = make([]bundler.EntryPoint, 0, len(buildOpts.EntryPoints)+len(buildOpts.EntryPointsAdvanced))
+	hasEntryPointWithWildcard := false
 	for _, ep := range buildOpts.EntryPoints {
 		entryPoints = append(entryPoints, bundler.EntryPoint{InputPath: ep})
+		if strings.ContainsRune(ep, '*') {
+			hasEntryPointWithWildcard = true
+		}
 	}
 	for _, ep := range buildOpts.EntryPointsAdvanced {
 		entryPoints = append(entryPoints, bundler.EntryPoint{InputPath: ep.InputPath, OutputPath: ep.OutputPath})
+		if strings.ContainsRune(ep.InputPath, '*') {
+			hasEntryPointWithWildcard = true
+		}
 	}
 	entryPointCount := len(entryPoints)
 	if buildOpts.Stdin != nil {
@@ -1344,7 +1355,7 @@ func validateBuildOptions(
 		}
 	}
 
-	if options.AbsOutputDir == "" && entryPointCount > 1 {
+	if options.AbsOutputDir == "" && (entryPointCount > 1 || hasEntryPointWithWildcard) {
 		log.AddError(nil, logger.Range{},
 			"Must use \"outdir\" when there are multiple input files")
 	} else if options.AbsOutputDir == "" && options.CodeSplitting {
@@ -1418,6 +1429,11 @@ func validateBuildOptions(
 		log.AddError(nil, logger.Range{}, "Splitting currently only works with the \"esm\" format")
 	}
 
+	// Code splitting is experimental and currently only enabled for ES6 modules
+	if options.TSConfigPath != "" && options.TSConfigRaw != "" {
+		log.AddError(nil, logger.Range{}, "Cannot provide \"tsconfig\" as both a raw string and a path")
+	}
+
 	// If we aren't writing the output to the file system, then we can allow the
 	// output paths to be the same as the input paths. This helps when serving.
 	if !buildOpts.Write {
@@ -1447,12 +1463,11 @@ type rebuildArgs struct {
 
 type rebuildState struct {
 	result    BuildResult
-	summary   buildSummary
 	watchData fs.WatchData
 	options   config.Options
 }
 
-func rebuildImpl(args rebuildArgs, oldSummary buildSummary) rebuildState {
+func rebuildImpl(args rebuildArgs, oldHashes map[string]string) (rebuildState, map[string]string) {
 	log := logger.NewStderrLog(args.logOptions)
 
 	// All validation warnings are repeated for every rebuild
@@ -1480,12 +1495,12 @@ func rebuildImpl(args rebuildArgs, oldSummary buildSummary) rebuildState {
 	}
 
 	// Scan over the bundle
-	bundle := bundler.ScanBundle(log, realFS, args.caches, args.entryPoints, args.options, timer)
+	bundle := bundler.ScanBundle(config.BuildCall, log, realFS, args.caches, args.entryPoints, args.options, timer)
 	watchData = realFS.WatchData()
 
 	// The new build summary remains the same as the old one when there are
 	// errors. A failed build shouldn't erase the previous successful build.
-	newSummary := oldSummary
+	newHashes := oldHashes
 
 	// Stop now if there were errors
 	if !log.HasErrors() {
@@ -1503,17 +1518,24 @@ func rebuildImpl(args rebuildArgs, oldSummary buildSummary) rebuildState {
 			result.Metafile = metafile
 
 			// Populate the results to return
+			var hashBytes [8]byte
 			result.OutputFiles = make([]OutputFile, len(results))
+			newHashes = make(map[string]string)
 			for i, item := range results {
 				if args.options.WriteToStdout {
 					item.AbsPath = "<stdout>"
 				}
+				hasher := xxhash.New()
+				hasher.Write(item.Contents)
+				binary.LittleEndian.PutUint64(hashBytes[:], hasher.Sum64())
+				hash := base64.RawStdEncoding.EncodeToString(hashBytes[:])
 				result.OutputFiles[i] = OutputFile{
 					Path:     item.AbsPath,
 					Contents: item.Contents,
+					Hash:     hash,
 				}
+				newHashes[item.AbsPath] = hash
 			}
-			newSummary = summarizeOutputFiles(result.OutputFiles)
 
 			// Write output files before "OnEnd" callbacks run so they can expect
 			// output files to exist on the file system. "OnEnd" callbacks can be
@@ -1532,8 +1554,8 @@ func rebuildImpl(args rebuildArgs, oldSummary buildSummary) rebuildState {
 				} else {
 					// Delete old files that are no longer relevant
 					var toDelete []string
-					for absPath := range oldSummary {
-						if _, ok := newSummary[absPath]; !ok {
+					for absPath := range oldHashes {
+						if _, ok := newHashes[absPath]; !ok {
 							toDelete = append(toDelete, absPath)
 						}
 					}
@@ -1546,7 +1568,7 @@ func rebuildImpl(args rebuildArgs, oldSummary buildSummary) rebuildState {
 							defer waitGroup.Done()
 							fs.BeforeFileOpen()
 							defer fs.AfterFileClose()
-							if oldHash, ok := oldSummary[result.AbsPath]; ok && oldHash == newSummary[result.AbsPath] {
+							if oldHash, ok := oldHashes[result.AbsPath]; ok && oldHash == newHashes[result.AbsPath] {
 								if contents, err := ioutil.ReadFile(result.AbsPath); err == nil && bytes.Equal(contents, result.Contents) {
 									// Skip writing out files that haven't changed since last time
 									return
@@ -1556,9 +1578,9 @@ func rebuildImpl(args rebuildArgs, oldSummary buildSummary) rebuildState {
 								log.AddError(nil, logger.Range{}, fmt.Sprintf(
 									"Failed to create output directory: %s", err.Error()))
 							} else {
-								var mode os.FileMode = 0644
+								var mode os.FileMode = 0666
 								if result.IsExecutable {
-									mode = 0755
+									mode = 0777
 								}
 								if err := ioutil.WriteFile(result.AbsPath, result.Contents, mode); err != nil {
 									log.AddError(nil, logger.Range{}, fmt.Sprintf(
@@ -1653,10 +1675,9 @@ func rebuildImpl(args rebuildArgs, oldSummary buildSummary) rebuildState {
 
 	return rebuildState{
 		result:    result,
-		summary:   newSummary,
 		options:   args.options,
 		watchData: watchData,
-	}
+	}, newHashes
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -1670,54 +1691,7 @@ func transformImpl(input string, transformOpts TransformOptions) TransformResult
 		LogLevel:      validateLogLevel(transformOpts.LogLevel),
 		Overrides:     validateLogOverrides(transformOpts.LogOverride),
 	})
-
-	// Settings from the user come first
-	var unusedImportFlagsTS config.UnusedImportFlagsTS
-	useDefineForClassFieldsTS := config.Unspecified
-	jsx := config.JSXOptions{
-		Preserve:         transformOpts.JSX == JSXPreserve,
-		AutomaticRuntime: transformOpts.JSX == JSXAutomatic,
-		Factory:          validateJSXExpr(log, transformOpts.JSXFactory, "factory"),
-		Fragment:         validateJSXExpr(log, transformOpts.JSXFragment, "fragment"),
-		Development:      transformOpts.JSXDev,
-		ImportSource:     transformOpts.JSXImportSource,
-		SideEffects:      transformOpts.JSXSideEffects,
-	}
-
-	// Settings from "tsconfig.json" override those
-	var tsTarget *config.TSTarget
-	var tsAlwaysStrict *config.TSAlwaysStrict
 	caches := cache.MakeCacheSet()
-	if transformOpts.TsconfigRaw != "" {
-		source := logger.Source{
-			KeyPath:    logger.Path{Text: "tsconfig.json"},
-			PrettyPath: "tsconfig.json",
-			Contents:   transformOpts.TsconfigRaw,
-		}
-		if result := resolver.ParseTSConfigJSON(log, source, &caches.JSONCache, nil); result != nil {
-			if result.JSX != config.TSJSXNone {
-				jsx.SetOptionsFromTSJSX(result.JSX)
-			}
-			if len(result.JSXFactory) > 0 {
-				jsx.Factory = config.DefineExpr{Parts: result.JSXFactory}
-			}
-			if len(result.JSXFragmentFactory) > 0 {
-				jsx.Fragment = config.DefineExpr{Parts: result.JSXFragmentFactory}
-			}
-			if len(result.JSXImportSource) > 0 {
-				jsx.ImportSource = result.JSXImportSource
-			}
-			if result.UseDefineForClassFields != config.Unspecified {
-				useDefineForClassFieldsTS = result.UseDefineForClassFields
-			}
-			unusedImportFlagsTS = config.UnusedImportFlagsFromTsconfigValues(
-				result.PreserveImportsNotUsedAsValues,
-				result.PreserveValueImports,
-			)
-			tsTarget = result.TSTarget
-			tsAlwaysStrict = result.TSAlwaysStrictOrStrict()
-		}
-	}
 
 	// Apply default values
 	if transformOpts.Sourcefile == "" {
@@ -1728,13 +1702,13 @@ func transformImpl(input string, transformOpts TransformOptions) TransformResult
 	}
 
 	// Convert and validate the transformOpts
-	targetFromAPI, jsFeatures, cssFeatures, targetEnv := validateFeatures(log, transformOpts.Target, transformOpts.Engines)
+	jsFeatures, cssFeatures, cssPrefixData, targetEnv := validateFeatures(log, transformOpts.Target, transformOpts.Engines)
 	jsOverrides, jsMask, cssOverrides, cssMask := validateSupported(log, transformOpts.Supported)
 	platform := validatePlatform(transformOpts.Platform)
 	defines, injectedDefines := validateDefines(log, transformOpts.Define, transformOpts.Pure, platform, false /* isBuildAPI */, false /* minify */, transformOpts.Drop)
 	mangleCache := cloneMangleCache(log, transformOpts.MangleCache)
 	options := config.Options{
-		TargetFromAPI:                      targetFromAPI,
+		CSSPrefixData:                      cssPrefixData,
 		UnsupportedJSFeatures:              jsFeatures.ApplyOverrides(jsOverrides, jsMask),
 		UnsupportedCSSFeatures:             cssFeatures.ApplyOverrides(cssOverrides, cssMask),
 		UnsupportedJSFeatureOverrides:      jsOverrides,
@@ -1742,39 +1716,46 @@ func transformImpl(input string, transformOpts TransformOptions) TransformResult
 		UnsupportedCSSFeatureOverrides:     cssOverrides,
 		UnsupportedCSSFeatureOverridesMask: cssMask,
 		OriginalTargetEnv:                  targetEnv,
-		TSTarget:                           tsTarget,
-		TSAlwaysStrict:                     tsAlwaysStrict,
-		JSX:                                jsx,
-		Defines:                            defines,
-		InjectedDefines:                    injectedDefines,
-		Platform:                           platform,
-		SourceMap:                          validateSourceMap(transformOpts.Sourcemap),
-		LegalComments:                      validateLegalComments(transformOpts.LegalComments, false /* bundle */),
-		SourceRoot:                         transformOpts.SourceRoot,
-		ExcludeSourcesContent:              transformOpts.SourcesContent == SourcesContentExclude,
-		OutputFormat:                       validateFormat(transformOpts.Format),
-		GlobalName:                         validateGlobalName(log, transformOpts.GlobalName),
-		MinifySyntax:                       transformOpts.MinifySyntax,
-		MinifyWhitespace:                   transformOpts.MinifyWhitespace,
-		MinifyIdentifiers:                  transformOpts.MinifyIdentifiers,
-		MangleProps:                        validateRegex(log, "mangle props", transformOpts.MangleProps),
-		ReserveProps:                       validateRegex(log, "reserve props", transformOpts.ReserveProps),
-		MangleQuoted:                       transformOpts.MangleQuoted == MangleQuotedTrue,
-		DropDebugger:                       (transformOpts.Drop & DropDebugger) != 0,
-		ASCIIOnly:                          validateASCIIOnly(transformOpts.Charset),
-		IgnoreDCEAnnotations:               transformOpts.IgnoreAnnotations,
-		TreeShaking:                        validateTreeShaking(transformOpts.TreeShaking, false /* bundle */, transformOpts.Format),
-		AbsOutputFile:                      transformOpts.Sourcefile + "-out",
-		KeepNames:                          transformOpts.KeepNames,
-		UseDefineForClassFields:            useDefineForClassFieldsTS,
-		UnusedImportFlagsTS:                unusedImportFlagsTS,
+		TSConfigRaw:                        transformOpts.TsconfigRaw,
+		JSX: config.JSXOptions{
+			Preserve:         transformOpts.JSX == JSXPreserve,
+			AutomaticRuntime: transformOpts.JSX == JSXAutomatic,
+			Factory:          validateJSXExpr(log, transformOpts.JSXFactory, "factory"),
+			Fragment:         validateJSXExpr(log, transformOpts.JSXFragment, "fragment"),
+			Development:      transformOpts.JSXDev,
+			ImportSource:     transformOpts.JSXImportSource,
+			SideEffects:      transformOpts.JSXSideEffects,
+		},
+		Defines:               defines,
+		InjectedDefines:       injectedDefines,
+		Platform:              platform,
+		SourceMap:             validateSourceMap(transformOpts.Sourcemap),
+		LegalComments:         validateLegalComments(transformOpts.LegalComments, false /* bundle */),
+		SourceRoot:            transformOpts.SourceRoot,
+		ExcludeSourcesContent: transformOpts.SourcesContent == SourcesContentExclude,
+		OutputFormat:          validateFormat(transformOpts.Format),
+		GlobalName:            validateGlobalName(log, transformOpts.GlobalName),
+		MinifySyntax:          transformOpts.MinifySyntax,
+		MinifyWhitespace:      transformOpts.MinifyWhitespace,
+		MinifyIdentifiers:     transformOpts.MinifyIdentifiers,
+		LineLimit:             transformOpts.LineLimit,
+		MangleProps:           validateRegex(log, "mangle props", transformOpts.MangleProps),
+		ReserveProps:          validateRegex(log, "reserve props", transformOpts.ReserveProps),
+		MangleQuoted:          transformOpts.MangleQuoted == MangleQuotedTrue,
+		DropLabels:            append([]string{}, transformOpts.DropLabels...),
+		DropDebugger:          (transformOpts.Drop & DropDebugger) != 0,
+		ASCIIOnly:             validateASCIIOnly(transformOpts.Charset),
+		IgnoreDCEAnnotations:  transformOpts.IgnoreAnnotations,
+		TreeShaking:           validateTreeShaking(transformOpts.TreeShaking, false /* bundle */, transformOpts.Format),
+		AbsOutputFile:         transformOpts.Sourcefile + "-out",
+		KeepNames:             transformOpts.KeepNames,
 		Stdin: &config.StdinInfo{
 			Loader:     validateLoader(transformOpts.Loader),
 			Contents:   input,
 			SourceFile: transformOpts.Sourcefile,
 		},
 	}
-	if options.Stdin.Loader == config.LoaderCSS {
+	if options.Stdin.Loader.IsCSS() {
 		options.CSSBanner = transformOpts.Banner
 		options.CSSFooter = transformOpts.Footer
 	} else {
@@ -1813,7 +1794,7 @@ func transformImpl(input string, transformOpts TransformOptions) TransformResult
 
 		// Scan over the bundle
 		mockFS := fs.MockFS(make(map[string]string), fs.MockUnix, "/")
-		bundle := bundler.ScanBundle(log, mockFS, caches, nil, options, timer)
+		bundle := bundler.ScanBundle(config.TransformCall, log, mockFS, caches, nil, options, timer)
 
 		// Stop now if there were errors
 		if !log.HasErrors() {
@@ -1903,8 +1884,10 @@ func importKindToResolveKind(kind ast.ImportKind) ResolveKind {
 		return ResolveJSDynamicImport
 	case ast.ImportRequireResolve:
 		return ResolveJSRequireResolve
-	case ast.ImportAt, ast.ImportAtConditional:
+	case ast.ImportAt:
 		return ResolveCSSImportRule
+	case ast.ImportComposesFrom:
+		return ResolveCSSComposesFrom
 	case ast.ImportURL:
 		return ResolveCSSURLToken
 	default:
@@ -1926,6 +1909,8 @@ func resolveKindToImportKind(kind ResolveKind) ast.ImportKind {
 		return ast.ImportRequireResolve
 	case ResolveCSSImportRule:
 		return ast.ImportAt
+	case ResolveCSSComposesFrom:
+		return ast.ImportComposesFrom
 	case ResolveCSSURLToken:
 		return ast.ImportURL
 	default:
@@ -2079,7 +2064,8 @@ func loadPlugins(initialOptions *BuildOptions, fs fs.FS, log logger.Log, caches
 
 			// Make a new resolver so it has its own log
 			log := logger.NewDeferLog(logger.DeferLogNoVerboseOrDebug, validateLogOverrides(initialOptions.LogOverride))
-			resolver := resolver.NewResolver(fs, log, caches, *optionsForResolve)
+			optionsClone := *optionsForResolve
+			resolver := resolver.NewResolver(config.BuildCall, fs, log, caches, &optionsClone)
 
 			// Make sure the resolve directory is an absolute path, which can fail
 			absResolveDir := validatePath(log, fs, options.ResolveDir, "resolve directory")
@@ -2113,7 +2099,7 @@ func loadPlugins(initialOptions *BuildOptions, fs fs.FS, log logger.Log, caches
 			result.Warnings = convertMessagesToPublic(logger.Warning, msgs)
 			if resolveResult != nil {
 				result.Path = resolveResult.PathPair.Primary.Text
-				result.External = resolveResult.IsExternal
+				result.External = resolveResult.PathPair.IsExternal
 				result.SideEffects = resolveResult.PrimarySideEffectsData == nil
 				result.Namespace = resolveResult.PathPair.Primary.Namespace
 				result.Suffix = resolveResult.PathPair.Primary.IgnoredSuffix
@@ -2496,21 +2482,6 @@ func analyzeMetafileImpl(metafile string, opts AnalyzeMetafileOptions) string {
 	return ""
 }
 
-type buildSummary map[string]uint64
-
-// This saves just enough information to be able to compute a useful diff
-// between two sets of output files. That way we don't need to hold both
-// sets of output files in memory at once to compute a diff.
-func summarizeOutputFiles(outputFiles []OutputFile) buildSummary {
-	summary := make(map[string]uint64)
-	for _, outputFile := range outputFiles {
-		hash := xxhash.New()
-		hash.Write(outputFile.Contents)
-		summary[outputFile.Path] = hash.Sum64()
-	}
-	return summary
-}
-
 func stripDirPrefix(path string, prefix string, allowedSlashes string) (string, bool) {
 	if strings.HasPrefix(path, prefix) {
 		pathLen := len(path)
